package ink.wgink.module.activiti.controller.api.oa;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.pojo.result.SuccessResult;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Map;

/**
 * @ClassName: OAFormReportController
 * @Description: OA表单上报
 * @Author: wanggeng
 * @Date: 2022/3/22 21:55
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "oa表单上报")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/oa-form-report")
public class OaFormReportController extends DefaultBaseController {

    @Autowired
    private IOaFormReportService oaFormReportService;

    @ApiOperation(value = "保存表单（发起流程）", notes = "保存表单（发起流程）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "processDefinitionId", value = "流程定义ID", paramType = "path"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path")
    })
    @PostMapping("save/definition-id/{processDefinitionId}/code/{formCode}/version/{formVersion}")
    public SuccessResult save(@PathVariable("processDefinitionId") String processDefinitionId,
                              @PathVariable("formCode") String formCode,
                              @PathVariable("formVersion") Integer formVersion,
                              @RequestBody Map<String, Object> params) {
        checkParams(params);
        oaFormReportService.save(processDefinitionId, formCode, formVersion, params);
        return new SuccessResult();
    }

    @ApiOperation(value = "更新表单", notes = "更新表单接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "taskId", value = "任务ID", paramType = "path"),
            @ApiImplicitParam(name = "formCode", value = "表单编码", paramType = "path"),
            @ApiImplicitParam(name = "formVersion", value = "表单版本", paramType = "path"),
            @ApiImplicitParam(name = "uid", value = "表单主键", paramType = "path")
    })
    @PutMapping("update/task-id/{taskId}/code/{formCode}/version/{formVersion}/uid/{uid}")
    public SuccessResult update(@PathVariable("taskId") String taskId,
                                @PathVariable("formCode") String formCode,
                                @PathVariable("formVersion") Integer formVersion,
                                @PathVariable("uid") String uid,
                                @RequestBody Map<String, Object> params) {
        checkParams(params);
        oaFormReportService.update(taskId, formCode, formVersion, uid, params);
        return new SuccessResult();
    }

    private void checkParams(Map<String, Object> body) {
        if (body.isEmpty()) {
            throw new ParamsException("提交内容不能为空");
        }
        for (Map.Entry<String, Object> kv : body.entrySet()) {
            String key = kv.getKey();
        }
    }
}
