package ink.wgink.module.activiti.service.activiti;

import com.alibaba.fastjson.JSONObject;
import ink.wgink.module.activiti.pojo.dtos.ActivitiFlowNodeDTO;
import ink.wgink.module.activiti.pojo.vos.ActivitiModelVO;
import org.activiti.bpmn.model.*;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IActivitiModelService
 * @Description: 流程模型
 * @Author: WangGeng
 * @Date: 2021/7/12 22:01
 * @Version: 1.0
 **/
public interface IActivitiModelService {

    /**
     * 排他网关条件关键字
     */
    String EXCLUSIVE_GATEWAY_CONDITION_KEY = "exc";

    /**
     * 并行网关条件关键字
     */
    String PARALLEL_GATEWAY_CONDITION_KEY = "par";

    /**
     * 保存模型
     *
     * @param modelId
     * @param activitiModelVO
     * @throws Exception
     */
    void save(String modelId, ActivitiModelVO activitiModelVO) throws Exception;

    /**
     * 删除模型
     *
     * @param modelId
     */
    void delete(String modelId);

    /**
     * 获取原生流程节点列表
     *
     * @param bpmnModel
     * @return
     */
    List<FlowNode> listFlowNode(BpmnModel bpmnModel);

    /**
     * 获取当前流程节点
     *
     * @param currentTaskDefinitionKey
     * @param processDefinitionId
     * @return
     */
    FlowNode getCurrentFlowNode(String currentTaskDefinitionKey, String processDefinitionId);

    /**
     * 获取下一个用户任务
     *
     * @param currentUserTask 当前用户任务
     * @return
     */
    UserTask getNextUserTask(UserTask currentUserTask);

    /**
     * 获取下一个用户任务
     *
     * @param exclusiveGateway 排他网关
     * @param reportForm       上报表单
     * @return
     */
    UserTask getNextUserTask(ExclusiveGateway exclusiveGateway, Map<String, Object> reportForm);

    /**
     * 获取下一个用户任务
     *
     * @param currentTaskDefinitionKey 当前任务定义Key
     * @param processDefinitionId      流程定义ID
     * @param reportForm               上报表单
     * @return
     */
    UserTask getNextUserTask(String currentTaskDefinitionKey, String processDefinitionId, Map<String, Object> reportForm);

    /**
     * 获得开始节点
     *
     * @param flowNodes
     * @return
     */
    StartEvent getStartEvent(List<FlowNode> flowNodes);

    /**
     * 获取自定义的流程节点列表
     *
     * @param flowNodes
     * @return
     */
    List<ActivitiFlowNodeDTO> listActivitiFlowNodeByModel(List<FlowNode> flowNodes);

    /**
     * 获取流程节点列表
     *
     * @param deploymentId 部署ID
     * @return
     */
    List<ActivitiFlowNodeDTO> listFlowNodeByDeploymentId(String deploymentId);

    /**
     * 获取编辑的json数据
     *
     * @param modelId
     * @return
     * @throws UnsupportedEncodingException
     */
    JSONObject getEditorJson(String modelId) throws UnsupportedEncodingException;

    /**
     * 获取模板套件
     *
     * @return
     * @throws IOException
     */
    JSONObject getStencilset() throws IOException;

    /**
     * 查看部署流程图
     *
     * @param response     响应
     * @param deploymentId 部署ID
     */
    void getProcessImage(HttpServletResponse response, String deploymentId);

    /**
     * 查看运行流程图
     *
     * @param response
     * @param processInstanceId
     */
    void getRuntimeProcessImage(HttpServletResponse response, String processInstanceId);

    /**
     * 获取流程xml
     *
     * @param processDefinitionId 流程定义ID
     */
    String getProcessXml(String processDefinitionId);

}
