package ink.wgink.module.activiti.service.activiti.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.service.activiti.IActivitiUserTaskService;
import org.activiti.bpmn.model.*;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.task.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: ActivitiUserTaskServiceImpl
 * @Description: 流程用户任务
 * @Author: wanggeng
 * @Date: 2022/3/29 17:44
 * @Version: 1.0
 */
@Service
public class ActivitiUserTaskServiceImpl extends DefaultBaseService implements IActivitiUserTaskService {

    @Autowired
    private TaskService taskService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private RepositoryService repositoryService;

    @Override
    public void getNextUserTask(String userTaskId) {
        Task task = taskService.createTaskQuery().taskId(userTaskId).singleResult();
        String taskDefinitionKey = task.getTaskDefinitionKey();
        String processInstanceId = task.getProcessInstanceId();
        String processDefinitionId = runtimeService.createProcessInstanceQuery().processInstanceId(processInstanceId).singleResult().getProcessDefinitionId();

        BpmnModel bpmnModel = repositoryService.getBpmnModel(processDefinitionId);

        FlowNode flowNode = (FlowNode) bpmnModel.getFlowElement(taskDefinitionKey);

        // 流入线
        List<SequenceFlow> incomingFlows = flowNode.getIncomingFlows();

        // 流出线
        List<SequenceFlow> outgoingFlows = flowNode.getOutgoingFlows();
        HashMap<String, Object> vars = new HashMap<>();
        vars.put("message", "");
        for (SequenceFlow outgoingFlow : outgoingFlows) {
            //获取输出节点元素
            FlowElement targetFlowElement = outgoingFlow.getTargetFlowElement();
            //排除非用户任务接点
            if (targetFlowElement instanceof UserTask) {
                LOG.debug("UserTask");
                UserTask userTask = (UserTask) targetFlowElement;
                System.out.println(userTask);
                //判断输出节点的el表达式
//                if (isCondition(outgoingFlow.getConditionExpression(), vars)) {
//                    //true 获取输出节点名称
//                    // nameList.add(outgoingFlow.getTargetFlowElement().getName());
//                }
            } else if (targetFlowElement instanceof ExclusiveGateway) {
                LOG.debug("排他网关");

            } else if (targetFlowElement instanceof ParallelGateway) {
                LOG.debug("并行网关");

            }
        }

        System.out.println(flowNode);
    }

    /**
     * el表达式判断
     *
     * @param expression
     * @param vars
     * @return
     */
    private static boolean isCondition(String expression, Map<String, Object> vars) {
        if (expression == null || expression == "") {
            return false;
        }

        //分割表达式
        String[] exprArr = expression.split("[{}$&]");
        for (String expr : exprArr) {
            //是否包含键message
            if (expr.contains("message")) {
                if (!vars.containsKey("message")) {
                    continue;
                }
                if (expr.contains("==")) {
                    String[] primes = expr.split("==");
                    String valExpr = primes[1].trim();
                    if (valExpr.startsWith("'")) {
                        valExpr = valExpr.substring(1);
                    }
                    if (valExpr.endsWith("'")) {
                        valExpr = valExpr.substring(0, valExpr.length() - 1);
                    }
                    if (primes.length == 2 && valExpr.equals(vars.get("message"))) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

}
