package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.dao.oa.INodeAssigneeDao;
import ink.wgink.module.activiti.pojo.vos.oa.nodemanage.config.AssigneeVO;
import ink.wgink.module.activiti.service.oa.INodeAssigneeService;
import ink.wgink.util.UUIDUtil;
import ink.wgink.util.map.HashMapUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Map;

/**
 * @ClassName: NodeAssigneeServiceImpl
 * @Description: 节点代理人
 * @Author: wanggeng
 * @Date: 2022/4/4 15:09
 * @Version: 1.0
 */
@Service
public class NodeAssigneeServiceImpl extends DefaultBaseService implements INodeAssigneeService {

    @Autowired
    private INodeAssigneeDao nodeAssigneeDao;

    @Override
    public void save(String modelId, Integer modelVersion, String nodeId, int nodeIndex, AssigneeVO assignee) {
        LOG.debug("新增代理人配置");
        Map<String, Object> params = HashMapUtil.beanToMap(assignee);
        params.put("nodeFieldId", UUIDUtil.getUUID());
        params.put("modelId", modelId);
        params.put("modelVersion", modelVersion);
        params.put("nodeId", nodeId);
        params.put("nodeIndex", nodeIndex);
        nodeAssigneeDao.save(params);
    }

    @Override
    public void update(String modelId, Integer modelVersion, String nodeId, int nodeIndex, AssigneeVO assignee) {
        delete(modelId, modelVersion, nodeId);
        save(modelId, modelVersion, nodeId, nodeIndex, assignee);
    }

    @Override
    public void updateDeploymentId(String modelId, Integer modelVersion, String deploymentId) {
        Map<String, Object> params = getHashMap(6);
        params.put("modelId", modelId);
        params.put("modelVersion", modelVersion);
        params.put("deploymentId", deploymentId);
        nodeAssigneeDao.updateDeploymentId(params);
    }

    @Override
    public void deleteDeploymentId(String deploymentId) {
        nodeAssigneeDao.deleteDeploymentId(deploymentId);
    }

    @Override
    public void deleteByModelIdAndModelVersion(String modelId, Integer modelVersion) {
        Map<String, Object> params = getHashMap(4);
        params.put("modelId", modelId);
        params.put("modelVersion", modelVersion);
        nodeAssigneeDao.delete(params);
    }

    /**
     * 删除
     *
     * @param modelId      模型ID
     * @param modelVersion 模型版本
     * @param nodeId       节点ID
     */
    private void delete(String modelId, Integer modelVersion, String nodeId) {
        LOG.debug("删除代理人配置");
        Map<String, Object> params = getHashMap(6);
        params.put("modelId", modelId);
        params.put("modelVersion", modelVersion);
        params.put("nodeId", nodeId);
        nodeAssigneeDao.delete(params);
    }

}
