package ink.wgink.module.activiti.service.oa.impl;

import ink.wgink.common.base.DefaultBaseService;
import ink.wgink.module.activiti.pojo.dtos.oa.OaHistoryTaskDTO;
import ink.wgink.module.activiti.pojo.dtos.oa.OaProcdefDTO;
import ink.wgink.module.activiti.pojo.dtos.oa.OaTaskDTO;
import ink.wgink.module.activiti.service.oa.IOaFormReportService;
import ink.wgink.module.activiti.service.oa.IOaService;
import ink.wgink.module.form.pojo.dtos.design.FormDTO;
import ink.wgink.module.form.service.design.IFormService;
import ink.wgink.pojo.result.SuccessResultList;
import org.activiti.engine.*;
import org.activiti.engine.history.HistoricTaskInstance;
import org.activiti.engine.history.HistoricTaskInstanceQuery;
import org.activiti.engine.repository.ProcessDefinition;
import org.activiti.engine.repository.ProcessDefinitionQuery;
import org.activiti.engine.task.Task;
import org.activiti.engine.task.TaskQuery;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: OaServiceImpl
 * @Description: oa管理
 * @Author: wanggeng
 * @Date: 2022/3/23 09:55
 * @Version: 1.0
 */
@Service
public class OaServiceImpl extends DefaultBaseService implements IOaService {

    @Autowired
    private RepositoryService repositoryService;
    @Autowired
    private RuntimeService runtimeService;
    @Autowired
    private FormService formService;
    @Autowired
    private TaskService taskService;
    @Autowired
    private HistoryService historyService;
    @Autowired
    private IFormService iFormService;

    @Override
    public List<OaProcdefDTO> listProcdef() {
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.list();
        return listOaProcdef(processDefinitions);
    }

    @Override
    public SuccessResultList<List<OaProcdefDTO>> listPageProcdef(int page, int rows) {
        ProcessDefinitionQuery processDefinitionQuery = repositoryService.createProcessDefinitionQuery();
        List<ProcessDefinition> processDefinitions = processDefinitionQuery.listPage(page - 1, rows);
        List<OaProcdefDTO> oaProcdefDTOs = listOaProcdef(processDefinitions);
        return new SuccessResultList<>(oaProcdefDTOs, page, processDefinitionQuery.count());
    }

    /**
     * 流程定义列表
     *
     * @param processDefinitions
     * @return
     */
    private List<OaProcdefDTO> listOaProcdef(List<ProcessDefinition> processDefinitions) {
        if (processDefinitions.isEmpty()) {
            return new ArrayList<>();
        }
        List<String> formIds = new ArrayList<>();
        List<OaProcdefDTO> oaProcdefDTOs = new ArrayList<>();
        processDefinitions.forEach(processDefinition -> {
            String formKey = formService.getStartFormData(processDefinition.getId()).getFormKey();
            processDefinition.getKey();
            OaProcdefDTO oaProcdefDTO = new OaProcdefDTO();
            oaProcdefDTO.setId(processDefinition.getId());
            oaProcdefDTO.setDeploymentId(processDefinition.getDeploymentId());
            oaProcdefDTO.setName(processDefinition.getName());
            oaProcdefDTO.setFormKey(formKey);
            oaProcdefDTOs.add(oaProcdefDTO);
            if (StringUtils.isBlank(formKey)) {
                return;
            }
            formIds.add(formKey);
        });
        List<FormDTO> formDTOs = iFormService.listByIds(formIds);
        oaProcdefDTOs.forEach(oaProcdefDTO -> {
            formDTOs.forEach(formDTO -> {
                if (!StringUtils.equals(oaProcdefDTO.getFormKey(), formDTO.getFormId())) {
                    return;
                }
                oaProcdefDTO.setFormCode(formDTO.getFormCode());
                oaProcdefDTO.setFormVersion(formDTO.getFormVersion());
            });
        });
        return oaProcdefDTOs;
    }

    @Override
    public List<OaTaskDTO> listTaskOfMine() {
        String userId = securityComponent.getCurrentUser().getUserId();
        TaskQuery taskQuery = taskService.createTaskQuery().taskCandidateOrAssigned(userId);
        List<Task> tasks = taskQuery.list();
        return listOaTask(tasks);
    }

    @Override
    public SuccessResultList<List<OaTaskDTO>> listPageTaskOfMine(int page, int rows) {
        String userId = securityComponent.getCurrentUser().getUserId();
        TaskQuery taskQuery = taskService.createTaskQuery().taskCandidateOrAssigned(userId);
        List<Task> tasks = taskQuery.listPage(page - 1, rows);
        List<OaTaskDTO> oaTaskDTOs = listOaTask(tasks);
        return new SuccessResultList<>(oaTaskDTOs, page, taskQuery.count());
    }

    /**
     * oa任务列表
     *
     * @param tasks
     * @return
     */
    private List<OaTaskDTO> listOaTask(List<Task> tasks) {
        List<OaTaskDTO> oaTaskDTOs = new ArrayList<>();
        tasks.forEach(task -> {
            OaTaskDTO oaTaskDTO = new OaTaskDTO();
            oaTaskDTOs.add(oaTaskDTO);

            oaTaskDTO.setTaskId(task.getId());
            oaTaskDTO.setTaskName(task.getName());
            oaTaskDTO.setTaskDescription(task.getDescription());
            oaTaskDTO.setOwner(task.getOwner());
            oaTaskDTO.setTaskDefinitionKey(task.getTaskDefinitionKey());
            oaTaskDTO.setProcessDefinitionId(task.getProcessDefinitionId());

            Map<String, Object> variables = runtimeService.getVariables(task.getProcessInstanceId());
            if (variables.isEmpty()) {
                return;
            }
            oaTaskDTO.setFormCode(variables.get(IOaFormReportService.KEY_FORM_CODE).toString());
            oaTaskDTO.setFormVersion(Integer.parseInt(variables.get(IOaFormReportService.KEY_FORM_VERSION).toString()));
            oaTaskDTO.setReportUid(variables.get(IOaFormReportService.KEY_REPORT_UID).toString());
        });
        return oaTaskDTOs;
    }

    @Override
    public List<OaHistoryTaskDTO> listHistoryTaskOfMine() {
        String userId = securityComponent.getCurrentUser().getUserId();
        HistoricTaskInstanceQuery historicTaskInstanceQuery = historyService.createHistoricTaskInstanceQuery().taskAssignee(userId);
        List<HistoricTaskInstance> historicTaskInstances = historicTaskInstanceQuery.list();
        return listOaHistoryTask(historicTaskInstances);
    }

    @Override
    public SuccessResultList<List<OaHistoryTaskDTO>> listPageHistoryTaskOfMine(int page, int rows) {
        String userId = securityComponent.getCurrentUser().getUserId();
        HistoricTaskInstanceQuery historicTaskInstanceQuery = historyService.createHistoricTaskInstanceQuery().taskAssignee(userId);
        List<HistoricTaskInstance> historicTaskInstances = historicTaskInstanceQuery.listPage(page - 1, rows);
        List<OaHistoryTaskDTO> oaHistoryTaskDTOs = listOaHistoryTask(historicTaskInstances);
        return new SuccessResultList<>(oaHistoryTaskDTOs, page, historicTaskInstanceQuery.count());
    }

    private List<OaHistoryTaskDTO> listOaHistoryTask(List<HistoricTaskInstance> historicTaskInstances) {
        List<OaHistoryTaskDTO> oaHistoryTaskDTOs = new ArrayList<>();
        historicTaskInstances.forEach(historicTaskInstance -> {
            OaHistoryTaskDTO oaHistoryTaskDTO = new OaHistoryTaskDTO();
            oaHistoryTaskDTO.setTaskId(historicTaskInstance.getId());
            oaHistoryTaskDTO.setTaskName(historicTaskInstance.getName());

        });
        return oaHistoryTaskDTOs;
    }

}
